package net.wanji.web.service.impl;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.RequiredArgsConstructor;
import net.wanji.common.enums.AreaEnum;
import net.wanji.common.enums.EventAlarmEnum;
import net.wanji.common.enums.EventAlarmSourceEnum;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.utils.tool.BeanListUtils;
import net.wanji.common.utils.tool.BeanMapUtils;
import net.wanji.common.utils.tool.CrossUtil;
import net.wanji.databus.dao.mapper.BaseAreaInfoMapper;
import net.wanji.databus.dao.mapper.BaseCrossInfoMapper;
import net.wanji.databus.dao.mapper.GreenwaveCrossMapper;
import net.wanji.databus.po.BaseAreaInfoPO;
import net.wanji.databus.po.BaseCrossInfoPO;
import net.wanji.databus.po.TBaseCrossInfo;
import net.wanji.databus.vo.*;
import net.wanji.feign.service.UtcFeignClients;
import net.wanji.web.common.enums.ControlHistEnum;
import net.wanji.web.common.enums.CrossAlarmEnum;
import net.wanji.web.common.enums.DeviceStatusEnum;
import net.wanji.web.common.enums.DeviceTypeEnum;
import net.wanji.web.common.util.StringUtils;
import net.wanji.web.common.util.date.DateStyle;
import net.wanji.web.common.util.date.DateUtils;
import net.wanji.web.dto.CrossIdNameDTO;
import net.wanji.web.entity.TBaseAreaCross;
import net.wanji.web.entity.TBaseAreaInfo;
import net.wanji.web.entity.TCrossControlHist;
import net.wanji.web.entity.TDeviceStatusInfo;
import net.wanji.web.mapper.*;
import net.wanji.web.po.*;
import net.wanji.web.service.SituationDetectionService;
import net.wanji.web.vo.situationDetection.*;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 态势监测服务类
 *
 * @author Kent HAN
 * @date 2022/10/26 13:48
 */
@Service
@RequiredArgsConstructor
public class SituationDetectionServiceImpl implements SituationDetectionService {

    private final OptRecordsMapper optRecordsMapper;
    private final CrossAlarmMapper crossAlarmMapper;
    private final EventAlarmMapper eventAlarmMapper;
    private final ControlHistMapper controlHistMapper;
    private final VehicleTraceMapper vehicleTraceMapper;
    private final VideoDeviceInfoMapper videoDeviceInfoMapper;
    private final GreenwaveRealtimeMapperOld greenwaveRealtimeMapperOld;
    private final AllDeviceStatusMapper allDeviceStatusMapper;
    private final GreenwaveCrossMapper greenwaveCrossMapper;
    private final BaseCrossInfoMapper tBaseBaseCrossInfoMapper;
    private final TBaseAreaInfoMapper tBaseAreaInfoMapper;
    private final TBaseAreaCrossMapper tBaseAreaCrossMapper;
    private final TDeviceStatusLogMapper tDeviceStatusLogMapper;
    private final UtcFeignClients utcFeignClients;
    private final BaseAreaInfoMapper baseAreaInfoMapper;
    private final TBaseAreaCrossMapper areaCrossMapper;
    private final BaseCrossInfoMapper crossInfoMapper;

    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    @Override
    public OptRecordsOutVO optRecords(String adCode, Date adaptTime) {
        OptRecordsOutVO optRecordsOutVO = new OptRecordsOutVO();
        List<OptRecordsOutVO.TrafficIndexVO> trafficIndexVOList = new ArrayList<>();

        List<BaseAreaDataPO> baseAreaDataPOList = optRecordsMapper.selectByAdCodeAndTime(adCode, adaptTime);
        double optNumSum = 0.0;
        double speedSum = 0.0;
        int speedCount = 0;
        for (BaseAreaDataPO baseAreaDataPO : baseAreaDataPOList) {
            // 加总优化次数
            Double optNum = baseAreaDataPO.getOptNum();
            optNumSum += optNum;

            Double speed = baseAreaDataPO.getSpeed();
            speedSum += speed;
            speedCount += 1;

            // 构造交通指数列表
            OptRecordsOutVO.TrafficIndexVO trafficIndexVO = new OptRecordsOutVO.TrafficIndexVO();
            Date adaptFullTime = baseAreaDataPO.getAdaptTime();
            String adaptFullTimeStr = sdf.format(adaptFullTime);
            // 返回小时分钟
            String timeStr = adaptFullTimeStr.substring(11, 16);
            trafficIndexVO.setTime(timeStr);
            Double trafficIndex = baseAreaDataPO.getTrafficIndex();
            trafficIndexVO.setIndex(trafficIndex);
            trafficIndexVOList.add(trafficIndexVO);
        }
        // 计算平均速度
        double averageSpeed = speedSum / speedCount;

        optRecordsOutVO.setOptTimes((int) optNumSum);
        optRecordsOutVO.setSpeed((int) averageSpeed);
        optRecordsOutVO.setTrafficIndexVOList(trafficIndexVOList);

        return optRecordsOutVO;
    }

    @Override
    public List<CrossAlarmOutVO> crossAlarm() {
        List<CrossAlarmPO> crossAlarmPOList = crossAlarmMapper.selectStatusNotZero();
        List<CrossAlarmOutVO> crossAlarmOutVOList = new ArrayList<>();
        for (CrossAlarmPO crossAlarmPO : crossAlarmPOList) {
            // 获取状态
            Integer isUnbalance = crossAlarmPO.getIsUnbalance();
            Integer isSpillover = crossAlarmPO.getIsSpillover();
            Integer isCongestion = crossAlarmPO.getIsCongestion();

            if (isUnbalance == 1) {
                String msg = CrossAlarmEnum.ONE.getMsg();
                Integer minuteDuration = crossAlarmPO.getUnbalanceTime();
                addCrossAlarmVO(crossAlarmOutVOList, crossAlarmPO, msg, minuteDuration);
            }
            if (isCongestion == 1) {
                String msg = CrossAlarmEnum.TWO.getMsg();
                Integer minuteDuration = crossAlarmPO.getCongestionTime();
                addCrossAlarmVO(crossAlarmOutVOList, crossAlarmPO, msg, minuteDuration);
            }
            if (isSpillover == 1) {
                String msg = CrossAlarmEnum.THREE.getMsg();
                Integer minuteDuration = crossAlarmPO.getSpilloverTime();
                addCrossAlarmVO(crossAlarmOutVOList, crossAlarmPO, msg, minuteDuration);
            }
            if ("4".equals(crossAlarmPO.getStatus())) {
                String msg = CrossAlarmEnum.FOUR.getMsg();
                Integer minuteDuration = crossAlarmPO.getDuration();
                addCrossAlarmVO(crossAlarmOutVOList, crossAlarmPO, msg, minuteDuration);
            }
        }
        CrossAlarmComparator crossAlarmComparator = new CrossAlarmComparator();
        crossAlarmOutVOList.sort(crossAlarmComparator);

        return crossAlarmOutVOList;
    }

    private void addCrossAlarmVO(List<CrossAlarmOutVO> crossAlarmOutVOList, CrossAlarmPO crossAlarmPO, String msg,
                                 Integer minuteDuration) {
        CrossAlarmOutVO crossAlarmOutVO = new CrossAlarmOutVO();
        // 根据路口ID获取路口名称
        String crossId = crossAlarmPO.getCrossId();
        TBaseCrossInfo tBaseCrossInfo = tBaseBaseCrossInfoMapper.selectByPrimaryKey(crossId);
        crossAlarmOutVO.setCrossName(tBaseCrossInfo.getName());
        // 从例如POINT(116.29959807 39.95929275)中获取经纬度
        String location = tBaseCrossInfo.getLocation();
        double[] lonLat = CrossUtil.getLonLat(location);
        crossAlarmOutVO.setLongitude(lonLat[0]);
        crossAlarmOutVO.setLatitude(lonLat[1]);

        crossAlarmOutVO.setCrossStatus(msg);
        // 发生时间
        Date date = DateUtil.date();
        Date startTime = DateUtil.offset(date, DateField.MINUTE, -minuteDuration);
        String timeStr = sdf.format(startTime);
        String substring = timeStr.substring(11, 16); // 截取小时分钟
        crossAlarmOutVO.setStartTime(substring);

        crossAlarmOutVO.setDuration(minuteDuration);
        crossAlarmOutVOList.add(crossAlarmOutVO);
    }

    @Override
    public List<EventAlarmOutVO> eventAlarm(Date todayTime) throws ParseException {
        List<EventAlarmPO> eventAlarmPOList = eventAlarmMapper.selectByTodayTime(todayTime);
        List<EventAlarmOutVO> eventAlarmOutVOList = new ArrayList<>();
        for (EventAlarmPO eventAlarmPO : eventAlarmPOList) {
            EventAlarmOutVO eventAlarmOutVO = new EventAlarmOutVO();
            eventAlarmOutVO.setEventDesc(eventAlarmPO.getEventDesc());
            // 事件类型
            Integer typeCode = eventAlarmPO.getType();
            String msgByCode = EventAlarmEnum.getMsgByCode(typeCode);
            eventAlarmOutVO.setType(msgByCode);

            Date endTime = eventAlarmPO.getEndTime();
            if (typeCode == 1 || typeCode == 2) {
                // 发生时间
                Date startTime = eventAlarmPO.getStartTime();
                String startTimeStr = sdf.format(startTime);
                String substring = startTimeStr.substring(11, 16);
                eventAlarmOutVO.setStartTime(substring);
                // 时长
                if (endTime == null) {
                    Date nowTime = DateUtil.date();

                    // todo 时间为演示用，待删除
                    Date parse = sdf.parse("2022-10-27 09:56:23");
                    nowTime = parse;

                    long between = DateUtil.between(startTime, nowTime, DateUnit.MINUTE);
                    int betweenInt = (int) between;
                    eventAlarmOutVO.setDuration(betweenInt);
                } else {
                    long between = DateUtil.between(startTime, endTime, DateUnit.MINUTE);
                    int betweenInt = (int) between;
                    eventAlarmOutVO.setDuration(betweenInt);
                }
            }
            if (typeCode == 3 || typeCode == 4) {
                // 时间范围
                Date startTime = eventAlarmPO.getStartTime();
                String startTimeStr = sdf.format(startTime);
                String startTimeSubStr = startTimeStr.substring(0, 16); // 截取到分钟
                String endTimeStr = sdf.format(endTime);
                String endTimeSubStr = endTimeStr.substring(0, 16); // 截取到分钟
                eventAlarmOutVO.setTimeSpan(startTimeSubStr + "~" + endTimeSubStr);
            }

            Integer sourceCode = eventAlarmPO.getSource();
            String sourceStr = EventAlarmSourceEnum.getMsgByCode(sourceCode);
            eventAlarmOutVO.setSource(sourceStr);
            eventAlarmOutVO.setCoordinate(eventAlarmPO.getWkt());

            eventAlarmOutVOList.add(eventAlarmOutVO);

        }
        return eventAlarmOutVOList;
    }

    @Override
    public List<ControlHistOutVO> controlHist(String crossId, String currentTime) throws ParseException {
        List<ControlHistOutVO> controlHistOutVOList = new ArrayList<>();
        Date todayDate = sdf.parse(currentTime);
        List<ControlHistPO> controlHistPOList = controlHistMapper.selectByCrossId(crossId, todayDate);
        // 替换时间字符串
        for (ControlHistPO controlHistPO : controlHistPOList) {
            String controlTime = controlHistPO.getControlTime();
            if (controlTime.endsWith(":0")) {
                controlTime += "0";
            }
            if (':' == controlTime.charAt(1)) {
                controlTime = "0" + controlTime;
            }
            controlHistPO.setControlTime(controlTime);
        }
        // 按时间聚合
        String tmpTime = "";
        Map<String, Integer> typeStat = new LinkedHashMap<>();
        // 初始化Map
        initTypeStat(typeStat);
        for (int i = 0; i < controlHistPOList.size(); i++) {
            ControlHistPO controlHistPO = controlHistPOList.get(i);
            String controlTime = controlHistPO.getControlTime();
            if (tmpTime.equals(controlTime)) {
                // 构造Map
                Integer controlType = controlHistPO.getControlType();
                String typeStr = ControlHistEnum.getMsgByCode(controlType);
                Integer typeCount = controlHistPO.getTypeCount();
                typeStat.put(typeStr, typeCount);
            } else if ("".equals(tmpTime)) {
                initTypeStat(typeStat);
                tmpTime = controlTime;
                i--;
            } else {
                // 构造一条记录
                Map<String, Integer> resTypeStat = new LinkedHashMap<>(typeStat);
                addControlHist(controlHistOutVOList, tmpTime, resTypeStat);

                initTypeStat(typeStat);
                tmpTime = controlTime;
                i--;
            }
        }
        // 构造最后一条记录
        addControlHist(controlHistOutVOList, tmpTime, typeStat);
        controlHistOutVOList.sort(Comparator.comparing(
                ControlHistOutVO::getTime, Comparator.nullsLast(Comparator.naturalOrder())));

        return controlHistOutVOList;
    }

    private void initTypeStat(Map<String, Integer> typeStat) {
        typeStat.clear();
        for (ControlHistEnum value : ControlHistEnum.values()) {
            typeStat.put(value.getMsg(), 0);
        }
    }

    private void addControlHist(List<ControlHistOutVO> controlHistOutVOList,
                                String tmpTime, Map<String, Integer> typeStat) {
        ControlHistOutVO controlHistOutVO = new ControlHistOutVO();
        controlHistOutVO.setTime(tmpTime);
        controlHistOutVO.setTypeStat(typeStat);
        controlHistOutVOList.add(controlHistOutVO);
    }

    @Override
    public List<VehicleTraceOutVO> vehicleTrace(String adCode, String currentTime) throws ParseException {
        List<VehicleTraceOutVO> vehicleTraceOutVOList = new ArrayList<>();
        Date todayDate = sdf.parse(currentTime);
        List<VehicleTracePO> vehicleTracePOList = vehicleTraceMapper.selectByAdCode(adCode, todayDate);
        // 替换时间字符串
        for (VehicleTracePO vehicleTracePO : vehicleTracePOList) {
            String traceTime = vehicleTracePO.getTraceTime();
            if (traceTime.endsWith(":0")) {
                traceTime += "0";
            }
            if (':' == traceTime.charAt(1)) {
                traceTime = "0" + traceTime;
            }
            vehicleTracePO.setTraceTime(traceTime);
        }
        for (VehicleTracePO vehicleTracePO : vehicleTracePOList) {
            VehicleTraceOutVO vehicleTraceOutVO = new VehicleTraceOutVO();
            vehicleTraceOutVO.setTraceTime(vehicleTracePO.getTraceTime());
            vehicleTraceOutVO.setVehicleCount(vehicleTracePO.getVehicleCount());
            vehicleTraceOutVOList.add(vehicleTraceOutVO);
        }
        vehicleTraceOutVOList.sort(Comparator.comparing(
                VehicleTraceOutVO::getTraceTime, Comparator.nullsLast(Comparator.naturalOrder())));

        return vehicleTraceOutVOList;
    }

    @Override
    public List<VideoDeviceInfoOutVO> videoDeviceInfo(String adCode) {
        List<VideoDeviceInfoPO> videoDeviceInfoPOList = videoDeviceInfoMapper.selectVideoDevices(adCode);
        List<VideoDeviceInfoOutVO> videoDeviceInfoOutVOList = new ArrayList<>();
        for (VideoDeviceInfoPO videoDeviceInfoPO : videoDeviceInfoPOList) {
            VideoDeviceInfoOutVO videoDeviceInfoOutVO = new VideoDeviceInfoOutVO();
            videoDeviceInfoOutVO.setCode(videoDeviceInfoPO.getCode());
            videoDeviceInfoOutVO.setName(videoDeviceInfoPO.getName());
            Integer status = videoDeviceInfoPO.getStatus();
            videoDeviceInfoOutVO.setStatus(DeviceStatusEnum.getMsgByCode(status));
            videoDeviceInfoOutVO.setLocation(videoDeviceInfoPO.getLocation());
            videoDeviceInfoOutVOList.add(videoDeviceInfoOutVO);
        }
        return videoDeviceInfoOutVOList;
    }

    @Override
    public List<GreenwaveRealtimeOutVO> greenwaveRealtime(String adCode, String currentTime) throws ParseException {
        Date todayDate = sdf.parse(currentTime);
        List<GreenwaveRealtimePO> greenwaveRealtimePOList = greenwaveRealtimeMapperOld.selectByAdCodeAndCurrentTime(adCode, todayDate);
        List<GreenwaveRealtimeOutVO> greenwaveRealtimeOutVOList = new ArrayList<>();
        for (GreenwaveRealtimePO greenwaveRealtimePO : greenwaveRealtimePOList) {
            GreenwaveRealtimeOutVO greenwaveRealtimeOutVO = new GreenwaveRealtimeOutVO();
            greenwaveRealtimeOutVO.setId(greenwaveRealtimePO.getGreenwaveId());
            BeanUtils.copyProperties(greenwaveRealtimePO, greenwaveRealtimeOutVO);
            greenwaveRealtimeOutVOList.add(greenwaveRealtimeOutVO);
        }

        return greenwaveRealtimeOutVOList;
    }

    @Override
    public Map<String, Map<String, Integer>> allDeviceStatus(String adCode) {
        TDeviceStatusInfo queryEntity = new TDeviceStatusInfo();
        if (StringUtils.isNotEmpty(adCode)) {
            queryEntity.setType(Integer.valueOf(adCode));
        }
        List<TDeviceStatusInfo> tDeviceStatusInfos = allDeviceStatusMapper.selectByEntity(queryEntity);
        Map<String, Map<String, Integer>> res = new LinkedHashMap<>();

        Map<Integer, List<TDeviceStatusInfo>> deviceTypeMap = tDeviceStatusInfos.stream().collect(Collectors.groupingBy(TDeviceStatusInfo::getType));
        for (Map.Entry<Integer, List<TDeviceStatusInfo>> entry : deviceTypeMap.entrySet()) {
            Integer type = entry.getKey();
            List<TDeviceStatusInfo> all = entry.getValue();
            List<TDeviceStatusInfo> offline = all.stream().filter(tDeviceStatusInfo -> tDeviceStatusInfo.getStatus() == 0).collect(Collectors.toList());
            List<TDeviceStatusInfo> normal = all.stream().filter(tDeviceStatusInfo -> isDealStatus(tDeviceStatusInfo)).collect(Collectors.toList());
            Map deviceMap = new HashMap<>();
            deviceMap.put(DeviceStatusEnum.NORMAL.getMsg(), normal.size());
            deviceMap.put(DeviceStatusEnum.WRONG.getMsg(), all.size() - normal.size() - offline.size());
            deviceMap.put(DeviceStatusEnum.OFFLINE.getMsg(), offline.size());
            res.put(DeviceTypeEnum.getMsgByCode(type), deviceMap);
        }

        return res;
    }

    @Override
    public List<CrossDeviceStatusInfoOutVO> crossDeviceStatusInfo() {
        List<CrossDeviceStatusInfoOutVO> results = tBaseBaseCrossInfoMapper.listCrossDeviceStatusInfo();
        return results;
    }

    @Override
    public BaseCrossInfoPO selectCrossInfoById(String signalId) {
        return tBaseBaseCrossInfoMapper.selectById(signalId);
    }

    @Override
    public List<JurisdictionAreaTreeVO> jurisdictionTree(Integer areaId, Integer type) throws Exception {
        List<JurisdictionAreaTreeVO> resultList = new ArrayList<>();
        // 查询辖区列表
        LambdaQueryWrapper<TBaseAreaInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TBaseAreaInfo::getType, type);
        if (Objects.nonNull(areaId)) {
            queryWrapper.eq(TBaseAreaInfo::getCode, areaId);
        }
        List<TBaseAreaInfo> tBaseAreaInfos = tBaseAreaInfoMapper.selectList(queryWrapper);

        // 通过辖区获取辖区内路口
        List<Integer> areadCodeList = tBaseAreaInfos.stream().map(TBaseAreaInfo::getCode).collect(Collectors.toList());
        LambdaQueryWrapper<TBaseAreaCross> crossQuery = new LambdaQueryWrapper<>();
        if (!CollectionUtils.isEmpty(areadCodeList)) {
            crossQuery.in(TBaseAreaCross::getAreaId, areadCodeList);
        }
        List<TBaseAreaCross> tBaseAreaCrosses = tBaseAreaCrossMapper.selectList(crossQuery);

        // 查询路口信息
        List<TBaseCrossInfo> tBaseCrossInfos = tBaseBaseCrossInfoMapper.selectList(new LambdaQueryWrapper<>());

        // 将路口和辖区转化为辖区实体
        getJurisdictionAreaTreeVO(resultList, tBaseAreaInfos, tBaseAreaCrosses, tBaseCrossInfos);

        // 构建树结构
        Map<String, List<JurisdictionAreaTreeVO>> map = resultList.stream().collect(Collectors.groupingBy(JurisdictionAreaTreeVO::getParentCode));
        for (JurisdictionAreaTreeVO jurisdictionAreaTreeVO : resultList) {
            for (Map.Entry<String, List<JurisdictionAreaTreeVO>> entry : map.entrySet()) {
                if (Objects.equals(jurisdictionAreaTreeVO.getId(), entry.getKey())) {
                    jurisdictionAreaTreeVO.setChlidren(entry.getValue());
                    break;
                }
            }
        }

        List<JurisdictionAreaTreeVO> result = resultList.stream().filter(vo -> StringUtils.equals("0", vo.getParentCode())).collect(Collectors.toList());

        return result;
    }

    /**
     * 构建返回所有辖区集合
     *
     * @param resultList
     * @param tBaseAreaInfos
     * @param tBaseAreaCrosses
     * @param tBaseCrossInfos
     */
    private static void getJurisdictionAreaTreeVO(List<JurisdictionAreaTreeVO> resultList, List<TBaseAreaInfo> tBaseAreaInfos,
                                                  List<TBaseAreaCross> tBaseAreaCrosses, List<TBaseCrossInfo> tBaseCrossInfos) {
        // 构建辖区实体
        for (TBaseAreaInfo parent : tBaseAreaInfos) {
            JurisdictionAreaTreeVO jurisdictionAreaVO = new JurisdictionAreaTreeVO();
            jurisdictionAreaVO.setId(String.valueOf(parent.getCode()));
            jurisdictionAreaVO.setName(parent.getName());
            jurisdictionAreaVO.setType("1");
            jurisdictionAreaVO.setPolylines(parent.getPolylines());
            jurisdictionAreaVO.setLevel(parent.getParentCode() == 0 ? 1 : null);
            jurisdictionAreaVO.setParentCode(String.valueOf(parent.getParentCode()));
            jurisdictionAreaVO.setChlidren(null);
            resultList.add(jurisdictionAreaVO);
        }
        // 普通路口转化为辖区实体
        List<String> areaCrossList = tBaseAreaCrosses.stream().map(TBaseAreaCross::getCrossId).collect(Collectors.toList());
        for (TBaseCrossInfo crossInfo : tBaseCrossInfos) {
            if (!areaCrossList.contains(crossInfo.getId())) {
                JurisdictionAreaTreeVO jurisdictionAreaVO = new JurisdictionAreaTreeVO();
                jurisdictionAreaVO.setId(String.valueOf(crossInfo.getId()));
                jurisdictionAreaVO.setName(crossInfo.getName());
                jurisdictionAreaVO.setType("2");
                String location = crossInfo.getLocation().replace("POINT(", "").replace(" ", ",").replace(")", "");
                jurisdictionAreaVO.setName(crossInfo.getName());
                jurisdictionAreaVO.setPoint(location);
                jurisdictionAreaVO.setLevel(0);
                jurisdictionAreaVO.setParentCode("0");
                jurisdictionAreaVO.setChlidren(null);
                resultList.add(jurisdictionAreaVO);
            }
        }
        // 将辖区内路口转化为辖区实体
        for (TBaseAreaInfo parent : tBaseAreaInfos) {
            Integer parentCode = parent.getCode();
            for (TBaseAreaCross tBaseAreaCross : tBaseAreaCrosses) {
                if (Objects.equals(parentCode, tBaseAreaCross.getAreaId())) {
                    JurisdictionAreaTreeVO signalAreaVO = new JurisdictionAreaTreeVO();
                    signalAreaVO.setId(tBaseAreaCross.getCrossId());
                    String name = null;
                    String location = null;
                    for (TBaseCrossInfo tBaseCrossInfo : tBaseCrossInfos) {
                        if (StringUtils.equals(tBaseCrossInfo.getId(), tBaseAreaCross.getCrossId())) {
                            name = tBaseCrossInfo.getName();
                            location = tBaseCrossInfo.getLocation();
                            if (location.contains("POINT")) {
                                location = location.replace("POINT(", "").replace(" ", ",").replace(")", "");
                            }
                            break;
                        }
                    }
                    signalAreaVO.setName(name);
                    signalAreaVO.setType("2");
                    signalAreaVO.setPoint(location);
                    signalAreaVO.setParentCode(String.valueOf(parentCode));
                    signalAreaVO.setChlidren(null);
                    resultList.add(signalAreaVO);
                }
            }
        }
    }

    @Override
    public List<AreaListVO> selectAreaList(Integer areaId) {
        List<AreaListVO> results = new ArrayList<>();

        // 通过辖区将路口分组
        LambdaQueryWrapper<TBaseAreaCross> queryWrapper = new LambdaQueryWrapper<>();
        if (Objects.nonNull(areaId)) {
            queryWrapper.eq(TBaseAreaCross::getAreaId, areaId);
        }
        List<TBaseAreaCross> entityList = tBaseAreaCrossMapper.selectList(queryWrapper);
        // key:辖区编号 value:路口列表
        Map<Integer, List<TBaseAreaCross>> areaIdMap = entityList.stream().collect(Collectors.groupingBy(TBaseAreaCross::getAreaId));

        // 路口编号获取路口名称和经纬度
        List<TBaseCrossInfo> tBaseCrossInfos = tBaseBaseCrossInfoMapper.selectList(new LambdaQueryWrapper<>());
        Map<String, String> crossIdNameMap = tBaseCrossInfos.stream().collect(Collectors.toMap(TBaseCrossInfo::getId, TBaseCrossInfo::getName));
        Map<String, String> crossIdLocationMap = tBaseCrossInfos.stream().collect(Collectors.toMap(TBaseCrossInfo::getId, TBaseCrossInfo::getLocation));

        // 辖区列表
        List<TBaseAreaInfo> jurisdictionAreaList = tBaseAreaInfoMapper.selectList(new LambdaQueryWrapper<>());

        // 构建区域返回列表
        for (Map.Entry<Integer, List<TBaseAreaCross>> entry : areaIdMap.entrySet()) {
            AreaListVO areaListVO = new AreaListVO();
            Set<CrossIdNameDTO> crossSet = new HashSet<>();
            Set<String> ployLines = new HashSet<>();
            List<TBaseAreaCross> crosseList = entry.getValue();
            String areaName = null;
            for (TBaseAreaCross tBaseAreaCross : crosseList) {
                for (TBaseAreaInfo tBaseAreaInfo : jurisdictionAreaList) {
                    if (Objects.equals(tBaseAreaCross.getAreaId(), tBaseAreaInfo.getCode())) {
                        areaName = tBaseAreaInfo.getName();
                        CrossIdNameDTO crossIdNameDTO = new CrossIdNameDTO();
                        crossIdNameDTO.setCrossId(tBaseAreaCross.getCrossId());
                        crossIdNameDTO.setCrossName(crossIdNameMap.get(tBaseAreaCross.getCrossId()));
                        String locationStr = crossIdLocationMap.get(tBaseAreaCross.getCrossId());
                        String location = locationStr.replace("POINT", "").replace("(", "").replace(")", "").replace(" ", ",");
                        crossIdNameDTO.setLocation(location);
                        crossSet.add(crossIdNameDTO);
                        ployLines.add(tBaseAreaInfo.getPolylines());
                    }
                }
            }
            areaListVO.setCrossInfos(crossSet);
            areaListVO.setPloyLines(ployLines);
            areaListVO.setAreaName(areaName);
            results.add(areaListVO);
        }

        return results;
    }

    @Override
    public List<SignalFaultInfoVO> listSignalFaultInfos(String dealStatus) throws Exception {
        TDeviceStatusInfo tDeviceStatusInfo = new TDeviceStatusInfo();
        tDeviceStatusInfo.setType(1); // 信号机
        List<TDeviceStatusInfo> tDeviceStatusInfos = allDeviceStatusMapper.selectByEntity(tDeviceStatusInfo);
        // 获取utc库中路口信息表的信号机ip
        List<CrossInfoOutVo> crossInfoOutVoList = getCrossInfoOutVoList(new CrossInfoPageVO());

        List<SignalFaultInfoVO> signalFaultInfoVOS = new ArrayList<>(tDeviceStatusInfos.size());
        tDeviceStatusInfos.forEach(tDeviceStatus -> {
            SignalFaultInfoVO signalFaultInfoVO = new SignalFaultInfoVO();
            signalFaultInfoVO.setDealStatus("1"); // 默认未处理
            BeanUtils.copyProperties(tDeviceStatus, signalFaultInfoVO);
            signalFaultInfoVO.setStartTime(DateUtils.dateToString(tDeviceStatus.getGmtCreate(), DateStyle.YYYY_MM_DD_HH_MM_SS));
            if (!CollectionUtils.isEmpty(crossInfoOutVoList)) {
                for (CrossInfoOutVo crossInfoOutVo : crossInfoOutVoList) {
                    if (StringUtils.equals(crossInfoOutVo.getId(), tDeviceStatus.getCode())) {
                        signalFaultInfoVO.setIp(crossInfoOutVo.getIp());
                        if (isDealStatus(tDeviceStatus)) {
                            signalFaultInfoVO.setDealStatus("2");
                        }
                    }
                }
            }
            signalFaultInfoVOS.add(signalFaultInfoVO);
        });
        List<SignalFaultInfoVO> result = null;
        if (StringUtils.isNotEmpty(dealStatus)) {
            result = signalFaultInfoVOS.stream()
                    .filter(signalFaultInfoVO -> StringUtils.equals(dealStatus, signalFaultInfoVO.getDealStatus())).collect(Collectors.toList());
        } else {
            result = signalFaultInfoVOS;
        }
        result = result.stream().sorted(Comparator.comparing(SignalFaultInfoVO::getDealStatus).reversed()
                .thenComparing(SignalFaultInfoVO::getGmtCreate).reversed()).collect(Collectors.toList());
        return result;
    }

    @Override
    public DeviceFaultCountVO countSignalFaultInfos() {
        TDeviceStatusInfo queryEntity = new TDeviceStatusInfo();
        queryEntity.setType(1);
        List<TDeviceStatusInfo> tDeviceStatusInfos = allDeviceStatusMapper.selectByEntity(queryEntity);
        List<TDeviceStatusInfo> dealList = tDeviceStatusInfos.stream().filter(tDeviceStatusInfo -> isDealStatus(tDeviceStatusInfo)).collect(Collectors.toList());
        int dealCount = dealList.size();
        int faultCount = tDeviceStatusInfos.size() - dealCount;

        DeviceFaultCountVO countVO = new DeviceFaultCountVO();
        countVO.setDealCount(dealCount);
        countVO.setFaultCount(faultCount);
        countVO.setAllCount(tDeviceStatusInfos.size());
        return countVO;
    }

    /**
     * 在线并且无故障为已处理
     *
     * @param tDeviceStatusInfo
     * @return
     */
    private static boolean isDealStatus(TDeviceStatusInfo tDeviceStatusInfo) {
        return tDeviceStatusInfo.getStatus() == 1 && tDeviceStatusInfo.getFaultType() == 0;
    }

    @Override
    public List<SignalOperationModeVO> listSignalControlList(Integer operationMode) {
        List<SignalOperationModeVO> signalOperationModeVOS = new ArrayList<>();
        LambdaQueryWrapper<TBaseCrossInfo> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(TBaseCrossInfo::getIsSignal, "1");
        List<TBaseCrossInfo> tBaseCrossInfos = tBaseBaseCrossInfoMapper.selectList(wrapper);
        List<TCrossControlHist> tCrossControlHists = controlHistMapper.selectRecentList();
        if (!CollectionUtils.isEmpty(tCrossControlHists) && !CollectionUtils.isEmpty(tBaseCrossInfos)) {
            List<String> isSignalControlList = tBaseCrossInfos.stream().map(TBaseCrossInfo::getId).collect(Collectors.toList());
            List<TCrossControlHist> filter = tCrossControlHists.stream().filter(vo -> isSignalControlList.contains(vo.getCrossId())).collect(Collectors.toList());
            if (Objects.nonNull(operationMode)) {
                if (operationMode == 1) { //中心优化
                    List<TCrossControlHist> centerOptList = filter.stream().filter(tCrossControlHist -> tCrossControlHist.getType() <= 5 || tCrossControlHist.getType() >= 1).collect(Collectors.toList());
                    BeanListUtils.populateList(centerOptList, signalOperationModeVOS, SignalOperationModeVO.class);
                }
                if (operationMode == 2) { // 手动控制
                    List<TCrossControlHist> manualControlList = filter.stream().filter(item -> item.getType() ==7).collect(Collectors.toList());
                    BeanListUtils.populateList(manualControlList, signalOperationModeVOS, SignalOperationModeVO.class);
                }
                if (operationMode == 3) { // 边缘优化
                    List<TCrossControlHist> edgeOptList = filter.stream().filter(tCrossControlHist -> tCrossControlHist.getType() == 6).collect(Collectors.toList());
                    BeanListUtils.populateList(edgeOptList, signalOperationModeVOS, SignalOperationModeVO.class);
                }
                if (operationMode == 4) { // 特勤控制
                    List<TCrossControlHist> specialServiceList = filter.stream().filter(tCrossControlHist -> tCrossControlHist.getType() == 8).collect(Collectors.toList());
                    BeanListUtils.populateList(specialServiceList, signalOperationModeVOS, SignalOperationModeVO.class);
                }
            } else {
                BeanListUtils.populateList(filter, signalOperationModeVOS, SignalOperationModeVO.class);
            }

            for (SignalOperationModeVO signalOperationModeVO : signalOperationModeVOS) {
                for (TBaseCrossInfo tBaseCrossInfo : tBaseCrossInfos) {
                    if (StringUtils.equals(signalOperationModeVO.getCrossId(), tBaseCrossInfo.getId())) {
                        // 转换经纬度格式
                        String location = tBaseCrossInfo.getLocation().replace("POINT", "").replace("(", "").replace(")", "").replace(" ", ",");
                        signalOperationModeVO.setLocation(location);
                        signalOperationModeVO.setCrossName(tBaseCrossInfo.getName());
                        signalOperationModeVO.setOperationTime(DateUtils.dateToString(signalOperationModeVO.getStartTime(), DateStyle.YYYY_MM_DD_HH_MM_SS));
                    }
                }
            }

        }
        List<SignalOperationModeVO> sort = signalOperationModeVOS.stream().sorted(Comparator.comparing(SignalOperationModeVO::getStartTime).reversed()).collect(Collectors.toList());
        return sort;
    }

    @Override
    public List<SignalOperationModeCountVO> countSignalControlInfos() {
        int centerOptCount = 0;
        int borderOptCount = 0;
        int manualOptCount = 0;
        int specialOptCount = 0;
        List<SignalOperationModeVO> signalOperationModeVOS = listSignalControlList(null);
        for (SignalOperationModeVO modeVO : signalOperationModeVOS) {
            Integer type = modeVO.getType();
            if (Objects.nonNull(type)) {
                if (type < 6) {
                    centerOptCount++;
                }
                if (type == 6) { // todo 提供测试数据，边缘优化还没有
                    borderOptCount++;
                }
                if (type == 7) {
                    manualOptCount++;
                }
                if (type == 8) { // todo 提供测试数据, 特勤
                    specialOptCount++;
                }
            }
        }
        List<SignalOperationModeCountVO> resultList = new ArrayList<>();
        List<Integer> typeArr = Arrays.asList(1, 2, 3, 4);
        List<Integer> countArr = Arrays.asList(centerOptCount, manualOptCount, borderOptCount, specialOptCount);
        for (int i = 0; i < 4; i++) {
            SignalOperationModeCountVO vo = new SignalOperationModeCountVO();
            vo.setType(typeArr.get(i));
            vo.setCount(countArr.get(i));
            resultList.add(vo);
        }
        return resultList;
    }

    @Override
    public List<SignalManufacturerCountInfoVO> signalManufacturerInfoCount() throws Exception {

        List<ManufacturerInfoOutVO> manufacturerInfoOutVOList = getManufacturerInfoOutVOList(new ManufacturerVO());
        List<CrossInfoOutVo> crossInfoOutVoList = getCrossInfoOutVoList(new CrossInfoPageVO());

        // key:nickName value:List<crossId>
        Map<String, List<String>> nickNameCrossIdMap = getCrossIdNickNameMap(manufacturerInfoOutVOList, crossInfoOutVoList);
        List<TDeviceStatusInfo> tDeviceStatusInfos = allDeviceStatusMapper.selectByEntity(new TDeviceStatusInfo());
        List<SignalManufacturerCountInfoVO> resultList = getResultList(nickNameCrossIdMap, tDeviceStatusInfos);

        return resultList;
    }

    @Override
    public List<OptAreaInfoVO> areaList() {
        List<OptAreaInfoVO> res = new ArrayList<>();
        List<BaseAreaInfoPO> baseAreaInfoPOList = baseAreaInfoMapper.selectAllArea(AreaEnum.TRANSPORT.getCode());
        for (BaseAreaInfoPO baseAreaInfoPO : baseAreaInfoPOList) {
            OptAreaInfoVO optAreaInfoVO = new OptAreaInfoVO();
            Integer areaId = baseAreaInfoPO.getId();
            optAreaInfoVO.setAreaId(areaId);
            optAreaInfoVO.setAreaName(baseAreaInfoPO.getName());
            optAreaInfoVO.setCrossInfos(buildCrossIdAndNameList(areaId));
            String polylines = baseAreaInfoPO.getPolylines();
            // 如果是道路，直接返回线
            if (Objects.equals(AreaEnum.ROAD.getCode(), baseAreaInfoPO.getType())) {
                optAreaInfoVO.setPloyLines(polylines);
            } else {
                List<String> wktList = buildWktList(polylines);
                optAreaInfoVO.setWktList(wktList);
            }
            res.add(optAreaInfoVO);
        }
        return res;
    }

    private List<String> buildWktList(String polylines) {
        String[] strArray = polylines.substring(1, polylines.length() - 1).split(", ");
        return Arrays.asList(strArray);
    }

    private List<OptCrossIdAndNameVO> buildCrossIdAndNameList(Integer areaId) {
        List<OptCrossIdAndNameVO> res = new ArrayList<>();

        List<String> crossIds = areaCrossMapper.selectCrossIdsByAreaId(areaId);
        List<BaseCrossInfoPO> baseCrossInfoPOList = crossInfoMapper.selectByCrossIds(crossIds);
        for (BaseCrossInfoPO baseCrossInfoPO : baseCrossInfoPOList) {
            OptCrossIdAndNameVO optCrossIdAndNameVO = new OptCrossIdAndNameVO();
            optCrossIdAndNameVO.setCrossId(baseCrossInfoPO.getId());
            optCrossIdAndNameVO.setCrossName(baseCrossInfoPO.getName());
            String location = baseCrossInfoPO.getLocation();
            double[] lonLat = CrossUtil.getLonLat(location);
            optCrossIdAndNameVO.setLocation(lonLat[0] + "," + lonLat[1]);
            res.add(optCrossIdAndNameVO);
        }
        return res;
    }

    /**
     * 统计不同厂商在线离线数量
     *
     * @param nickNameCrossIdMap
     * @param tDeviceStatusInfos
     * @return
     */
    private static List<SignalManufacturerCountInfoVO> getResultList(Map<String, List<String>> nickNameCrossIdMap, List<TDeviceStatusInfo> tDeviceStatusInfos) {
        List<SignalManufacturerCountInfoVO> resultList = new ArrayList<>();
        for (Map.Entry<String, List<String>> entry : nickNameCrossIdMap.entrySet()) {
            SignalManufacturerCountInfoVO signalManufacturerCountInfoVO = new SignalManufacturerCountInfoVO();
            String nickName = entry.getKey();
            List<String> crossIdList = entry.getValue();
            Integer onlineCount = 0;
            Integer offlineCount = 0;
            Integer faultCount = 0;
            signalManufacturerCountInfoVO.setName(nickName);
            for (TDeviceStatusInfo tDeviceStatusInfo : tDeviceStatusInfos) {
                String crossId = tDeviceStatusInfo.getCode();
                if (crossIdList.contains(crossId)) {
                    int status = tDeviceStatusInfo.getStatus();
                    if (status == 0) {
                        offlineCount++;
                    }
                    if (status == 1) {
                        onlineCount++;
                    }
                    if (status == 2) {
                        faultCount++;
                    }
                }
            }
            if (onlineCount > 0) {
                signalManufacturerCountInfoVO.setOnlineCount(onlineCount);
            }
            if (offlineCount > 0) {
                signalManufacturerCountInfoVO.setOfflineCount(offlineCount);
            }
            if (faultCount > 0) {
                signalManufacturerCountInfoVO.setFaultCount(faultCount);
            }
            resultList.add(signalManufacturerCountInfoVO);
        }
        return resultList;
    }

    /**
     * 统计当前路口归属厂商列表
     *
     * @param manufacturerInfoOutVOList
     * @param crossInfoOutVoList
     * @return
     */
    private Map<String, List<String>> getCrossIdNickNameMap(List<ManufacturerInfoOutVO> manufacturerInfoOutVOList, List<CrossInfoOutVo> crossInfoOutVoList) {
        Map<String, List<String>> crossIdNickNameMap = new HashMap<>();
        for (ManufacturerInfoOutVO manufacturerInfoOutVO : manufacturerInfoOutVOList) {
            String nickName = manufacturerInfoOutVO.getNickName();
            Integer id = manufacturerInfoOutVO.getId();
            String crossId = null;
            List<String> crossIdList = new ArrayList<>();
            for (CrossInfoOutVo crossInfoOutVo : crossInfoOutVoList) {
                crossId = crossInfoOutVo.getId();
                Integer manufacturerId = Integer.valueOf(crossInfoOutVo.getManufacturerId());
                if (Objects.equals(id, manufacturerId)) {
                    crossIdList.add(crossId);
                }
            }
            crossIdNickNameMap.put(nickName, crossIdList);
        }
        return crossIdNickNameMap;
    }

    /**
     * 通过utcFeignClients获取信号机详细信息
     *
     * @param crossInfoPageVO
     * @return List<CrossInfoOutVo>
     * @throws Exception
     */
    private List<CrossInfoOutVo> getCrossInfoOutVoList(CrossInfoPageVO crossInfoPageVO) throws Exception {
        JsonViewObject jsonViewObject = utcFeignClients.listCrossInfo(crossInfoPageVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            throw new Exception("远程调用UTC服务路口信息失败！");
        }
        Map resultMap = (Map) jsonViewObject.getContent();
        List<Map<String, Object>> list = (List<Map<String, Object>>) resultMap.get("list");
        List<CrossInfoOutVo> resultList = new ArrayList<>(list.size());
        for (Map<String, Object> map : list) {
            CrossInfoOutVo crossInfoVOResult = BeanMapUtils.mapToBean(map, CrossInfoOutVo.class);
            resultList.add(crossInfoVOResult);
        }
        return resultList;
    }

    /**
     * 通过utcFeignClients获取厂商详细信息
     *
     * @param manufacturerVO
     * @return List<ManufacturerInfoOutVO>
     * @throws Exception
     */
    private List<ManufacturerInfoOutVO> getManufacturerInfoOutVOList(ManufacturerVO manufacturerVO) throws Exception {
        JsonViewObject jsonViewObject = utcFeignClients.listManufacturerInfo(manufacturerVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            throw new Exception("远程调用UTC服务信号机厂商信息失败！");
        }
        List<ManufacturerInfoOutVO> list = new ArrayList<>();
        Map map = (Map) jsonViewObject.getContent();
        List<Map<String, Object>> content = (List<Map<String, Object>>) map.get("list");
        for (Map<String, Object> entry : content) {
            ManufacturerInfoOutVO manufacturerInfoOutVO = BeanMapUtils.mapToBean(entry, ManufacturerInfoOutVO.class);
            list.add(manufacturerInfoOutVO);
        }
        return list;
    }
}


class CrossAlarmComparator implements Comparator<CrossAlarmOutVO> {

    @Override
    public int compare(CrossAlarmOutVO o1, CrossAlarmOutVO o2) {
        // 先按路口名称排序，再按溢出-拥堵-失衡排序
        int flag = o1.getCrossName().compareTo(o2.getCrossName());
        if (flag == 0) {
            String crossStatus1 = o1.getCrossStatus();
            Integer codeInt1 = CrossAlarmEnum.getCodeIntByMsg(crossStatus1);
            String crossStatus2 = o2.getCrossStatus();
            Integer codeInt2 = CrossAlarmEnum.getCodeIntByMsg(crossStatus2);
            return codeInt2 - codeInt1;
        }
        return flag;
    }


}
