package net.wanji.web.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import net.wanji.common.framework.exception.RidException;
import net.wanji.common.gts.GeometryUtil;
import net.wanji.common.gts.GeomsConvertUtil;
import net.wanji.common.gts.service.GtsService;
import net.wanji.common.utils.tool.CrossUtil;
import net.wanji.databus.dao.entity.RidInfoEntity;
import net.wanji.databus.dao.mapper.BaseCrossInfoMapper;
import net.wanji.databus.dao.mapper.CrossDirDataRealtimeMapper;
import net.wanji.databus.dao.mapper.RidInfoMapper;
import net.wanji.databus.po.BaseCrossInfoPO;
import net.wanji.databus.vo.LockControlVO;
import net.wanji.feign.service.UtcFeignClients;
import net.wanji.web.bo.*;
import net.wanji.web.common.enums.CrossDirEnum;
import net.wanji.web.kafka.ConsumerHandler;
import net.wanji.web.mapper.*;
import net.wanji.web.po.SpecialServiceCrossPO;
import net.wanji.web.po.SpecialServicePO;
import net.wanji.web.service.SpecialServiceService;
import net.wanji.web.vo.specialService.*;
import net.wanji.web.vo.specialService.SpecialServiceDetailVO.CrossListElement;
import org.jetbrains.annotations.NotNull;
import org.locationtech.jts.geom.Geometry;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author Kent HAN
 * @date 2022/11/8 11:07
 */
@Service
public class SpecialServiceServiceImpl implements SpecialServiceService {
    @Autowired
    SpecialServiceMapper specialServiceMapper;
    @Autowired
    SpecialServiceCrossMapper specialServiceCrossMapper;
    @Autowired
    CustomRidInfoMapper customRidInfoMapper;
    @Autowired
    BaseCrossInfoMapper tBaseBaseCrossInfoMapper;
    @Autowired
    CrossDirTurnMapper crossDirTurnMapper;
    @Autowired
    CrossPhasePlanMapper crossPhasePlanMapper;
    @Autowired
    private BaseCrossInfoMapper baseCrossInfoMapper;
    @Autowired
    private RidInfoMapper ridInfoMapper;

    private Gson gson = new GsonBuilder().setDateFormat("yyyy-MM-dd HH:mm:ss").create();
    @Autowired
    private GtsService gtsService;
    @Autowired
    private CrossDirDataRealtimeMapper crossDirDataRealtimeMapper;
    @Qualifier("net.wanji.feign.service.UtcFeignClients")
    @Autowired
    private UtcFeignClients utcFeignClients;

    @Override
    @Transactional
    public void addOrUpdateSpecialService(AddSpecialServiceBO addSpecialServiceBO) {
        // 校验特勤名称是否重复
        checkSpecialServiceName(addSpecialServiceBO);
        
        Integer specialServiceId = addSpecialServiceBO.getId();
        String plateNum = addSpecialServiceBO.getPlateNum();
        if (ObjectUtil.isEmpty(specialServiceId)) { // 不传ID为新增
            // 插入特勤表
            SpecialServicePO specialServicePO = new SpecialServicePO();
            specialServicePO.setName(addSpecialServiceBO.getName());
            specialServicePO.setPlateNum(plateNum);
            specialServicePO.setStartLocation(addSpecialServiceBO.getStartLocation());
            specialServicePO.setEndLocation(addSpecialServiceBO.getEndLocation());

            specialServiceMapper.insertOne(specialServicePO);
            Integer specialServiceIdGenerated = specialServicePO.getId();

            // 插入特勤路口关系表
            List<RouteElementVO> route = addSpecialServiceBO.getRoute();

            for (int i = 0; i < route.size(); i++) {
                RouteElementVO routeElementVO = route.get(i);
                String startName = routeElementVO.getStartName();
                BaseCrossInfoPO startCross = baseCrossInfoMapper.selectByName(startName);
                String startCrossId = startCross.getId();
                String endName = routeElementVO.getEndName();
                BaseCrossInfoPO endCross = baseCrossInfoMapper.selectByName(endName);
                String endCrossId = endCross.getId();


                RidInfoEntity ridInfoEntity = ridInfoMapper.selectByStartEnd(startCrossId, endCrossId);
                if (i != route.size() - 1) { // 非结束路段
                    SpecialServiceCrossPO specialServiceCrossPO = new SpecialServiceCrossPO();
                    specialServiceCrossPO.setSpecialServiceId(specialServiceIdGenerated);
                    specialServiceCrossPO.setCrossId(startCrossId);
                    // 计算方向
                    if (i == 0) { // 开始路口
                        specialServiceCrossPO.setInDir(0);
                        Integer outDir = ridInfoEntity.getOutDir();
                        specialServiceCrossPO.setOutDir(outDir);
                    } else { // 中间路口
                        // 获取上一个路段
                        RouteElementVO preRoute = route.get(i - 1);
                        String preStartName = preRoute.getStartName();
                        BaseCrossInfoPO preStartCross = baseCrossInfoMapper.selectByName(preStartName);
                        String preStartCrossId = preStartCross.getId();

                        String preEndName = preRoute.getEndName();
                        BaseCrossInfoPO preEndCross = baseCrossInfoMapper.selectByName(preEndName);
                        String preEndCrossId = preEndCross.getId();
                        RidInfoEntity preRid = ridInfoMapper.selectByStartEnd(preStartCrossId, preEndCrossId);
                        // 当前路口驶入方向
                        Integer inDir = preRid.getInDir();
                        // 当前路口使出方向
                        Integer outDir = ridInfoEntity.getOutDir();
                        specialServiceCrossPO.setInDir(inDir);
                        specialServiceCrossPO.setOutDir(outDir);
                    }
                    specialServiceCrossPO.setSort(i + 1);
                    specialServiceCrossPO.setStatus(0);
                    specialServiceCrossPO.setAutoUnlock(0);
                    specialServiceCrossMapper.insertOne(specialServiceCrossPO);
                } else { // 结束路段
                    // 开始路口
                    SpecialServiceCrossPO specialServiceCrossPO = new SpecialServiceCrossPO();
                    specialServiceCrossPO.setSpecialServiceId(specialServiceIdGenerated);
                    specialServiceCrossPO.setCrossId(startCrossId);

                    RidInfoEntity preRid = null;
                    if (route.size() == 1) { // 如果只有两个路口
                        preRid = ridInfoEntity;
                    } else {
                        // 获取上一个路段
                        RouteElementVO preRoute = route.get(i - 1);
                        String preStartName = preRoute.getStartName();
                        BaseCrossInfoPO preStartCross = baseCrossInfoMapper.selectByName(preStartName);
                        String preStartCrossId = preStartCross.getId();

                        String preEndName = preRoute.getEndName();
                        BaseCrossInfoPO preEndCross = baseCrossInfoMapper.selectByName(preEndName);
                        String preEndCrossId = preEndCross.getId();
                        preRid = ridInfoMapper.selectByStartEnd(preStartCrossId, preEndCrossId);
                    }

                    // 当前路口驶入方向
                    Integer inDir = preRid.getInDir();
                    // 当前路口使出方向
                    Integer outDir = ridInfoEntity.getOutDir();
                    specialServiceCrossPO.setInDir(inDir);
                    specialServiceCrossPO.setOutDir(outDir);
                    specialServiceCrossPO.setSort(i + 1);
                    specialServiceCrossPO.setStatus(0);
                    specialServiceCrossPO.setAutoUnlock(0);
                    specialServiceCrossMapper.insertOne(specialServiceCrossPO);
                    // 结束路口
                    SpecialServiceCrossPO specialServiceCrossPOEnd = new SpecialServiceCrossPO();
                    specialServiceCrossPOEnd.setSpecialServiceId(specialServiceIdGenerated);
                    specialServiceCrossPOEnd.setCrossId(endCrossId);
                    Integer inDirEnd = ridInfoEntity.getInDir();
                    specialServiceCrossPOEnd.setInDir(inDirEnd);
                    specialServiceCrossPOEnd.setOutDir(0);
                    specialServiceCrossPOEnd.setSort(i + 2);
                    specialServiceCrossPOEnd.setStatus(0);
                    specialServiceCrossPOEnd.setAutoUnlock(0);
                    specialServiceCrossMapper.insertOne(specialServiceCrossPOEnd);
                }
            }
        } else { // 传ID为修改
            // 修改特勤表
            SpecialServicePO specialServicePO = new SpecialServicePO();
            specialServicePO.setId(specialServiceId);
            specialServicePO.setName(addSpecialServiceBO.getName());
            specialServicePO.setPlateNum(plateNum);
            specialServicePO.setStartLocation(addSpecialServiceBO.getStartLocation());
            specialServicePO.setEndLocation(addSpecialServiceBO.getEndLocation());

            specialServiceMapper.updateOne(specialServicePO);

            // 修改特勤路口关系表
            specialServiceCrossMapper.deleteCrossBySpecialServiceId(specialServiceId);
            List<RouteElementVO> route = addSpecialServiceBO.getRoute();

            for (int i = 0; i < route.size(); i++) {
                RouteElementVO routeElementVO = route.get(i);
                String startName = routeElementVO.getStartName();
                BaseCrossInfoPO startCross = baseCrossInfoMapper.selectByName(startName);
                String startCrossId = startCross.getId();
                String endName = routeElementVO.getEndName();
                BaseCrossInfoPO endCross = baseCrossInfoMapper.selectByName(endName);
                String endCrossId = endCross.getId();

                RidInfoEntity ridInfoEntity = ridInfoMapper.selectByStartEnd(startCrossId, endCrossId);
                if (i != route.size() - 1) { // 非结束路段
                    SpecialServiceCrossPO specialServiceCrossPO = new SpecialServiceCrossPO();
                    specialServiceCrossPO.setSpecialServiceId(specialServiceId);
                    specialServiceCrossPO.setCrossId(startCrossId);
                    // 计算方向
                    if (i == 0) { // 开始路口
                        specialServiceCrossPO.setInDir(0);
                        Integer outDir = ridInfoEntity.getOutDir();
                        specialServiceCrossPO.setOutDir(outDir);
                    } else { // 中间路口
                        // 获取上一个路段
                        RouteElementVO preRoute = route.get(i - 1);
                        String preStartName = preRoute.getStartName();
                        BaseCrossInfoPO preStartCross = baseCrossInfoMapper.selectByName(preStartName);
                        String preStartCrossId = preStartCross.getId();

                        String preEndName = preRoute.getEndName();
                        BaseCrossInfoPO preEndCross = baseCrossInfoMapper.selectByName(preEndName);
                        String preEndCrossId = preEndCross.getId();
                        RidInfoEntity preRid = ridInfoMapper.selectByStartEnd(preStartCrossId, preEndCrossId);
                        // 当前路口驶入方向
                        Integer inDir = preRid.getInDir();
                        // 当前路口使出方向
                        Integer outDir = ridInfoEntity.getOutDir();
                        specialServiceCrossPO.setInDir(inDir);
                        specialServiceCrossPO.setOutDir(outDir);
                    }
                    specialServiceCrossPO.setSort(i + 1);
                    specialServiceCrossPO.setStatus(0);
                    specialServiceCrossPO.setAutoUnlock(0);
                    specialServiceCrossMapper.insertOne(specialServiceCrossPO);
                } else { // 结束路段
                    // 开始路口
                    SpecialServiceCrossPO specialServiceCrossPO = new SpecialServiceCrossPO();
                    specialServiceCrossPO.setSpecialServiceId(specialServiceId);
                    specialServiceCrossPO.setCrossId(startCrossId);

                    RidInfoEntity preRid = null;
                    if (route.size() == 1) { // 如果只有两个路口
                        preRid = ridInfoEntity;
                    } else {
                        // 获取上一个路段
                        RouteElementVO preRoute = route.get(i - 1);
                        String preStartName = preRoute.getStartName();
                        BaseCrossInfoPO preStartCross = baseCrossInfoMapper.selectByName(preStartName);
                        String preStartCrossId = preStartCross.getId();

                        String preEndName = preRoute.getEndName();
                        BaseCrossInfoPO preEndCross = baseCrossInfoMapper.selectByName(preEndName);
                        String preEndCrossId = preEndCross.getId();
                        preRid = ridInfoMapper.selectByStartEnd(preStartCrossId, preEndCrossId);
                    }

                    // 当前路口驶入方向
                    Integer inDir = preRid.getInDir();
                    // 当前路口使出方向
                    Integer outDir = ridInfoEntity.getOutDir();
                    specialServiceCrossPO.setInDir(inDir);
                    specialServiceCrossPO.setOutDir(outDir);
                    specialServiceCrossPO.setSort(i + 1);
                    specialServiceCrossPO.setStatus(0);
                    specialServiceCrossPO.setAutoUnlock(0);
                    specialServiceCrossMapper.insertOne(specialServiceCrossPO);
                    // 结束路口
                    SpecialServiceCrossPO specialServiceCrossPOEnd = new SpecialServiceCrossPO();
                    specialServiceCrossPOEnd.setSpecialServiceId(specialServiceId);
                    specialServiceCrossPOEnd.setCrossId(endCrossId);
                    Integer inDirEnd = ridInfoEntity.getInDir();
                    specialServiceCrossPOEnd.setInDir(inDirEnd);
                    specialServiceCrossPOEnd.setOutDir(0);
                    specialServiceCrossPOEnd.setSort(i + 2);
                    specialServiceCrossPOEnd.setStatus(0);
                    specialServiceCrossPOEnd.setAutoUnlock(0);
                    specialServiceCrossMapper.insertOne(specialServiceCrossPOEnd);
                }
            }
        }
    }

    private void checkSpecialServiceName(AddSpecialServiceBO addSpecialServiceBO) {
        String specialServiceName = addSpecialServiceBO.getName();
        SpecialServicePO specialServicePO = specialServiceMapper.selectByName(specialServiceName);
        if (ObjectUtil.isNotEmpty(specialServicePO)) {
            throw new RuntimeException("特勤名称已存在");
        }
    }

    @Override
    public List<ListSpecialServicesVO> listSpecialServices() {
        List<ListSpecialServicesVO> res = new ArrayList<>();

        List<SpecialServicePO> specialServicePOList = specialServiceMapper.selectAll();
        for (SpecialServicePO specialServicePO : specialServicePOList) {
            ListSpecialServicesVO listSpecialServicesVO = new ListSpecialServicesVO();
            listSpecialServicesVO.setId(specialServicePO.getId());
            listSpecialServicesVO.setName(specialServicePO.getName());
            listSpecialServicesVO.setStatus(specialServicePO.getStatus());
            listSpecialServicesVO.setGmtModified(specialServicePO.getGmtModified());
            listSpecialServicesVO.setPlateNum(specialServicePO.getPlateNum());

            res.add(listSpecialServicesVO);
        }
        return res;
    }

    @Override
    public void updateSpecialService(UpdateSpecialServiceBO updateSpecialServiceBO) {
        specialServiceMapper.updateSpecialService(updateSpecialServiceBO);
    }

    @Override
    @Transactional
    public void deleteSpecialService(Integer id) {
        specialServiceMapper.deleteSpecialService(id);
        specialServiceCrossMapper.deleteCrossBySpecialServiceId(id);
    }

    private String getCrossIn(String crossInId) {
        // 从 13FNK0C6790_1_2_1 中获取 “北进口”
        int length = crossInId.length();
        String dir = crossInId.substring(length - 5, length - 4);
        String dirStr = CrossDirEnum.getMsgByCode(Integer.parseInt(dir));
        String s = dirStr + "进口";
        return s;
    }

    @Override
    public List<List<RouteElementVO>> specialServiceRoute(SpecialServiceRouteBO specialServiceRouteBO) {
        List<RidInfoEntity> ridInfoEntities = ridInfoMapper.selectAll();
        // 获取起点经纬度与rid连线相交的路段
        String startLonLat = specialServiceRouteBO.getStartLonLat();
        Geometry startGeometry = GeometryUtil.str2Geometry(startLonLat);
        Set<String> startRidPoints = new HashSet<>();
        for (RidInfoEntity ridInfoEntity : ridInfoEntities) {
            String ridLine = ridInfoEntity.getWkt();
            Geometry ridLineGeometry = GeometryUtil.str2Geometry(ridLine);
            Geometry ridLineBuf = GeomsConvertUtil.getGeometryBuf(ridLineGeometry, "50.0");
            boolean isIntersect = ridLineBuf.intersects(startGeometry);
            if (isIntersect) {
                String startCrossId = ridInfoEntity.getStartCrossId();
                String endCrossId = ridInfoEntity.getEndCrossId();
                startRidPoints.add(startCrossId);
                startRidPoints.add(endCrossId);
            }
        }
        // 获取起点路口
        double startMinDistance = Double.MAX_VALUE;
        BaseCrossInfoPO startCross = null;
        for (String ridPoint : startRidPoints) {
            BaseCrossInfoPO baseCrossInfoPO = baseCrossInfoMapper.selectById(ridPoint);
            String location = baseCrossInfoPO.getLocation();
            double[] lonLat = CrossUtil.getLonLat(location);
            String ridPointLonLat = lonLat[0] + "," + lonLat[1];
            double distance = gtsService.distance(startLonLat, ridPointLonLat);
            startMinDistance = Math.min(startMinDistance, distance);
            if (distance == startMinDistance) {
                startCross = baseCrossInfoPO;
            }
        }

        // 获取终点经纬度与rid连线相交的路段
        String endLonLat = specialServiceRouteBO.getEndLonLat();
        Geometry endGeometry = GeometryUtil.str2Geometry(endLonLat);
        Set<String> endRidPoints = new HashSet<>();
        for (RidInfoEntity ridInfoEntity : ridInfoEntities) {
            String ridLine = ridInfoEntity.getWkt();
            Geometry ridLineGeometry = GeometryUtil.str2Geometry(ridLine);
            Geometry ridLineBuf = GeomsConvertUtil.getGeometryBuf(ridLineGeometry, "50.0");
            boolean isIntersect = ridLineBuf.intersects(endGeometry);
            if (isIntersect) {
                String startCrossId = ridInfoEntity.getStartCrossId();
                String endCrossId = ridInfoEntity.getEndCrossId();
                endRidPoints.add(startCrossId);
                endRidPoints.add(endCrossId);
            }
        }
        // 获取终点路口
        double endMinDistance = Double.MAX_VALUE;
        BaseCrossInfoPO endCross = null;
        for (String ridPoint : endRidPoints) {
            BaseCrossInfoPO baseCrossInfoPO = baseCrossInfoMapper.selectById(ridPoint);
            String location = baseCrossInfoPO.getLocation();
            double[] lonLat = CrossUtil.getLonLat(location);
            String ridPointLonLat = lonLat[0] + "," + lonLat[1];
            double distance = gtsService.distance(endLonLat, ridPointLonLat);
            endMinDistance = Math.min(endMinDistance, distance);
            if (distance == endMinDistance) {
                endCross = baseCrossInfoPO;
            }
        }

        if (ObjectUtil.isNotEmpty(startCross) && ObjectUtil.isNotEmpty(endCross)
                && ObjectUtil.notEqual(startCross, endCross)) {
            List<List<RouteElementVO>> res = new ArrayList<>();
            // 构造第一条备选线路
            List<BaseCrossInfoPO> routeList1 = getRouteList(startCross, endCross);
            List<RouteElementVO> route1 = buildRoute(routeList1);
            // 构造第二条备选线路
            String startCrossId = startCross.getId();
            String endCrossLonLatStr = getCrossLonLatStr(endCross);
            List<RidInfoEntity> ridUpCrossList = ridInfoMapper.selectUpCross(startCrossId);
            Map<Double, BaseCrossInfoPO> distanceMap = new HashMap<>();
            List<Double> distanceList = new ArrayList<>();
            for (RidInfoEntity ridUpCross : ridUpCrossList) {
                String upCrossId = ridUpCross.getStartCrossId();
                BaseCrossInfoPO upCross = baseCrossInfoMapper.selectById(upCrossId);
                String upCrossLonLatStr = getCrossLonLatStr(upCross);
                double distance = gtsService.distance(endCrossLonLatStr, upCrossLonLatStr);
                distanceMap.put(distance, upCross);
                distanceList.add(distance);
            }
            Collections.sort(distanceList);
            BaseCrossInfoPO secondStartCross = null;
            try {
                secondStartCross = distanceMap.get(distanceList.get(1));
            } catch (IndexOutOfBoundsException e) {
                throw new RuntimeException("路网数据不连续");
            }
            List<BaseCrossInfoPO> routeList2 = new ArrayList<>();
            routeList2.add(startCross);
            getSecondRouteList(startCross, secondStartCross, endCross, routeList2);
            List<RouteElementVO> route2 = buildRoute(routeList2);

            res.add(route1);
            res.add(route2);
            return res;
        }
        return null;
    }

    private void getSecondRouteList(BaseCrossInfoPO startCross, BaseCrossInfoPO secondStartCross,
                                    BaseCrossInfoPO endCross, List<BaseCrossInfoPO> routeList2) {
        String endCrossLonLatStr = getCrossLonLatStr(endCross);
        // 获取上游路口到终点距离最近的
        BaseCrossInfoPO nextStartCross = secondStartCross;
        while (ObjectUtil.notEqual(endCross.getId(), nextStartCross.getId())) {
            String nextCrossId = nextStartCross.getId();
            double minDistance = Double.MAX_VALUE;
            List<RidInfoEntity> ridUpCrossList = ridInfoMapper.selectUpCross(nextStartCross.getId());
            for (RidInfoEntity ridUpCross : ridUpCrossList) {
                String upCrossId = ridUpCross.getStartCrossId();
                if (StrUtil.equals(upCrossId, startCross.getId())){
                    continue;
                }
                BaseCrossInfoPO upCross = baseCrossInfoMapper.selectById(upCrossId);
                String upCrossLonLatStr = getCrossLonLatStr(upCross);
                double distance = gtsService.distance(endCrossLonLatStr, upCrossLonLatStr);
                minDistance = Math.min(minDistance, distance);
                if (minDistance == distance && !hasCross(routeList2, upCross)) { // 已选过的路口不可再选
                    nextStartCross = upCross;
                }
            }
            if (ObjectUtil.equal(nextStartCross.getId(), nextCrossId)) {
                throw new RuntimeException("路网数据不连续");
            }
            routeList2.add(nextStartCross);
        }
        routeList2.add(1, secondStartCross);
    }

    private boolean hasCross(List<BaseCrossInfoPO> routeList, BaseCrossInfoPO upCross) {
        for (BaseCrossInfoPO baseCrossInfoPO : routeList) {
            String id = baseCrossInfoPO.getId();
            String upCrossId = upCross.getId();
            if (ObjectUtil.equal(id, upCrossId)) {
                return true;
            }
        }
        return false;
    }

    @Override
    public void isValidPoint(IsValidPointBO isValidPointBO) {
        String lonLat = isValidPointBO.getLonLat();
        List<RidInfoEntity> ridInfoEntities = ridInfoMapper.selectAll();
        // 获取点位经纬度与 rid 连线相交的路段
        Geometry geometry = GeometryUtil.str2Geometry(lonLat);
        Set<String> startRidPoints = new HashSet<>();
        for (RidInfoEntity ridInfoEntity : ridInfoEntities) {
            String ridLine = ridInfoEntity.getWkt();
            Geometry ridLineGeometry = GeometryUtil.str2Geometry(ridLine);
            Geometry ridLineBuf = GeomsConvertUtil.getGeometryBuf(ridLineGeometry, "50.0");
            boolean isIntersect = ridLineBuf.intersects(geometry);
            if (isIntersect) {
                String startCrossId = ridInfoEntity.getStartCrossId();
                String endCrossId = ridInfoEntity.getEndCrossId();
                startRidPoints.add(startCrossId);
                startRidPoints.add(endCrossId);
            }
        }
        if (CollectionUtil.isEmpty(startRidPoints)) {
            throw new RuntimeException("不是有效点位");
        }
    }

    @Override
    public SpecialServiceDetailVO specialServiceDetail(SpecialServiceIdBO specialServiceIdBO) {
        SpecialServiceDetailVO specialServiceDetailVO = new SpecialServiceDetailVO();
        Integer specialServiceId = specialServiceIdBO.getId();
        SpecialServicePO specialServicePO = specialServiceMapper.selectById(specialServiceId);

        specialServiceDetailVO.setName(specialServicePO.getName());
        // 构造路口列表
        List<SpecialServiceDetailVO.CrossListElement> crossList = buildCrossList(specialServiceId);
        specialServiceDetailVO.setCrossList(crossList);
        // 构造路线列表
        List<BaseCrossInfoPO> baseCrossInfoPOList = new ArrayList<>();
        for (CrossListElement crossListElement : crossList) {
            String crossId = crossListElement.getCrossId();
            BaseCrossInfoPO baseCrossInfoPO = baseCrossInfoMapper.selectById(crossId);
            baseCrossInfoPOList.add(baseCrossInfoPO);
        }
        List<RouteElementVO> route = buildRoute(baseCrossInfoPOList);
        specialServiceDetailVO.setRoute(route);
        return specialServiceDetailVO;
    }

    @Override
    @Transactional
    public void updateSpecialServiceDetail(UpdateSpecialServiceBO updateSpecialServiceBO) {
        // 验证是否有空方向
        checkValidCrosses(updateSpecialServiceBO);

        Integer specialServiceId = updateSpecialServiceBO.getSpecialServiceId();
        specialServiceCrossMapper.deleteCrossBySpecialServiceId(specialServiceId);
        String initCrossId = updateSpecialServiceBO.getInitCrossId();
        String finalCrossId = updateSpecialServiceBO.getFinalCrossId();
        List<CrossListElement> crossList = updateSpecialServiceBO.getCrossList();

        String currentCrossId = initCrossId;
        Integer sort = 1;
        Set<String> visitedCrossIds = new HashSet<>(); // 用于验证已经过的路口不可再经过
        while (!StrUtil.equals(finalCrossId, currentCrossId)) { // 未到最后一个路口
            if (visitedCrossIds.contains(currentCrossId)) {
                throw new RidException("路线规划不正确或路网数据不连续");
            }
            for (CrossListElement crossListElement : crossList) {
                String crossId = crossListElement.getCrossId();
                if (StrUtil.equals(currentCrossId, crossId) && !StrUtil.equals(currentCrossId, finalCrossId)) {
                    SpecialServiceCrossPO specialServiceCrossPO = new SpecialServiceCrossPO();
                    specialServiceCrossPO.setSpecialServiceId(specialServiceId);
                    specialServiceCrossPO.setCrossId(currentCrossId);

                    // 根据起点路口和使出方向确定路段
                    String routeEnd = crossListElement.getRouteEnd();
                    if (null == routeEnd) {
                        throw new RidException("路线规划不正确或路网数据不连续");
                    }
                    String outDirStr = routeEnd.charAt(routeEnd.length() - 1) + "";
                    Integer outDir = CrossDirEnum.getCodeByMsg(outDirStr);
                    RidInfoEntity ridInfoEntity = ridInfoMapper.selectByStartOurDir(currentCrossId, outDir);
                    if (ObjectUtil.isEmpty(ridInfoEntity)) {
                        throw new RidException("路线规划不正确或路网数据不连续");
                    }
                    String nextCrossId = ridInfoEntity.getEndCrossId();

                    if (StrUtil.equals(currentCrossId, initCrossId)) { // 是起点路口
                        specialServiceCrossPO.setInDir(0);
                        specialServiceCrossPO.setOutDir(outDir);
                    } else { // 中间路口
                        String routeStart = crossListElement.getRouteStart();
                        if (null == routeStart) {
                            throw new RidException("路线规划不正确或路网数据不连续");
                        }
                        String inDirStr = routeStart.charAt(routeStart.length() - 1) + "";
                        Integer inDir = CrossDirEnum.getCodeByMsg(inDirStr);
                        specialServiceCrossPO.setInDir(inDir);
                        specialServiceCrossPO.setOutDir(outDir);
                    }
                    specialServiceCrossPO.setSort(sort);
                    specialServiceCrossPO.setStatus(crossListElement.getStatus());
                    specialServiceCrossPO.setAutoUnlock(0);
                    specialServiceCrossMapper.insertOne(specialServiceCrossPO);
                    visitedCrossIds.add(currentCrossId);
                    currentCrossId = nextCrossId;
                    // 验证currentCrossId是否在规划路线里
                    checkCurrentCrossId(currentCrossId, crossList);
                    sort++;
                }
            }
        }
        // 最后一个路口
        for (CrossListElement crossListElement : crossList) {
            String crossId = crossListElement.getCrossId();
            if (StrUtil.equals(finalCrossId, crossId)) {
                SpecialServiceCrossPO specialServiceCrossPO = new SpecialServiceCrossPO();
                specialServiceCrossPO.setSpecialServiceId(specialServiceId);
                specialServiceCrossPO.setCrossId(finalCrossId);

                String routeStart = crossListElement.getRouteStart();
                if (null == routeStart) {
                    throw new RidException("路线规划不正确或路网数据不连续");
                }
                String inDirStr = routeStart.charAt(routeStart.length() - 1) + "";
                Integer inDir = CrossDirEnum.getCodeByMsg(inDirStr);

                specialServiceCrossPO.setInDir(inDir);
                specialServiceCrossPO.setOutDir(0);
                specialServiceCrossPO.setSort(sort);
                specialServiceCrossPO.setStatus(crossListElement.getStatus());
                specialServiceCrossPO.setAutoUnlock(0);
                specialServiceCrossMapper.insertOne(specialServiceCrossPO);
            }
        }
        // 验证路口总数
        List<SpecialServiceCrossPO> crossPOList =
                specialServiceCrossMapper.selectBySpecialServiceId(specialServiceId);
        if (crossPOList.size() != crossList.size()) {
            throw new RidException("路线规划不正确或路网数据不连续");
        }

    }

    private void checkCurrentCrossId(String currentCrossId, List<CrossListElement> crossList) {
        List<String> crossIdList = new ArrayList<>();
        for (CrossListElement crossListElement : crossList) {
            String crossId = crossListElement.getCrossId();
            crossIdList.add(crossId);
        }
        if (!crossIdList.contains(currentCrossId)) {
            throw new RidException("路线规划不正确或路网数据不连续");
        }
    }

    private void checkValidCrosses(UpdateSpecialServiceBO updateSpecialServiceBO) {
        List<CrossListElement> crossList = updateSpecialServiceBO.getCrossList();
        for (CrossListElement crossListElement : crossList) {
            String routeEnd = crossListElement.getRouteEnd();
            String routeStart = crossListElement.getRouteStart();
            if (StrUtil.isEmpty(routeEnd) || StrUtil.isEmpty(routeStart)) {
                throw new RidException("路线规划不正确或路网数据不连续");
            }
        }
    }

    @Override
    public List<SpecialDirListVO> getInDirList(List<CrossIdBO> crossIdBOList) {
        List<SpecialDirListVO> res =  new ArrayList<>();
        for (CrossIdBO crossIdBO : crossIdBOList) {
            SpecialDirListVO specialDirListVO = new SpecialDirListVO();
            String crossId = crossIdBO.getCrossId();
            specialDirListVO.setCrossId(crossId);
            List<String> specialDirList = buildInSpecialDirList(crossId);
            specialDirListVO.setSpecialDirList(specialDirList);
            res.add(specialDirListVO);
        }
        return res;
    }

    private List<String> buildInSpecialDirList(String crossId) {
        List<String> specialDirList = new ArrayList<>();
        specialDirList.add("起点");
        List<RidInfoEntity> ridList = ridInfoMapper.selectByEndCross(crossId);
        for (RidInfoEntity ridInfoEntity : ridList) {
            String name = ridInfoEntity.getName(); // 湘江中路:白沙路@劳动西路路段
            String streetName = name.split(":")[0];
            Integer inDir = ridInfoEntity.getInDir();
            String msgByCode = CrossDirEnum.getMsgByCode(inDir);
            specialDirList.add(streetName + msgByCode);
        }
        return specialDirList;
    }

    @Override
    public List<SpecialDirListVO> getOutDirList(List<CrossIdBO> crossIdBOList) {
        List<SpecialDirListVO> res =  new ArrayList<>();
        for (CrossIdBO crossIdBO : crossIdBOList) {
            SpecialDirListVO specialDirListVO = new SpecialDirListVO();
            String crossId = crossIdBO.getCrossId();
            specialDirListVO.setCrossId(crossId);
            List<String> specialDirList = buildOutSpecialDirList(crossId);
            specialDirListVO.setSpecialDirList(specialDirList);
            res.add(specialDirListVO);
        }
        return res;
    }

    private List<String> buildOutSpecialDirList(String crossId) {
        List<String> res = new ArrayList<>();
        res.add("终点");
        List<RidInfoEntity> ridList = ridInfoMapper.selectByStartCross(crossId);
        for (RidInfoEntity ridInfoEntity : ridList) {
            String name = ridInfoEntity.getName(); // 湘江中路:白沙路@劳动西路路段
            String streetName = name.split(":")[0];
            Integer outDir = ridInfoEntity.getOutDir();
            String msgByCode = CrossDirEnum.getMsgByCode(outDir);
            res.add(streetName + msgByCode);
        }
        return res;
    }

    @Override
    @Transactional
    public void batchLock(BatchLockBO batchLockBO) {
        Integer specialServiceId = batchLockBO.getSpecialServiceId();
        List<LockControlVO> lockControlVOList = batchLockBO.getLockControlVOList();

        for (LockControlVO lockControlVO : lockControlVOList) {
            utcFeignClients.lockControl(lockControlVO);
            Integer command = lockControlVO.getCommand();
            String crossId = lockControlVO.getCrossCode();
            if (1 == command) { // 锁定
                specialServiceCrossMapper.updateStatusSuccess(crossId);
                // 将关联特勤改为执行中
                specialServiceMapper.updateStatusEnable(specialServiceId);
            } else if (0 == command) { // 解锁
                specialServiceCrossMapper.updateStatusFail(crossId, specialServiceId);
            }
        }
        // 如果特勤路口都是解锁，则更新特勤状态为未执行
        List<SpecialServiceCrossPO> crossList = specialServiceCrossMapper.selectBySpecialServiceId(specialServiceId);
        for (SpecialServiceCrossPO cross : crossList) {
            Integer status = cross.getStatus();
            if (1 == status) {
                return;
            }
        }
        specialServiceMapper.updateStatusDisable(specialServiceId);
    }

    @Override
    public GroupListVO groupList(SpecialServiceIdBO specialServiceIdBO) {
        GroupListVO groupListVO = new GroupListVO();

        Integer specialServiceId = specialServiceIdBO.getId();
        SpecialServicePO specialServicePO = specialServiceMapper.selectById(specialServiceId);
        groupListVO.setSpecialServiceId(specialServiceId);

        List<GroupListVO.GroupListElement> groupList = buildGroupList(specialServicePO);
        groupListVO.setGroupList(groupList);

        return groupListVO;
    }

    @Override
    @Transactional
    public void updateGroup(GroupListVO groupListVO) {
        // 校验分组名称不能重复
        List<GroupListVO.GroupListElement> groupList = groupListVO.getGroupList();
        Set<String> groupNameSet = new HashSet<>();
        for (GroupListVO.GroupListElement groupListElement : groupList) {
            String groupName = groupListElement.getGroupName();
            if (!groupNameSet.add(groupName)) {
                throw new RuntimeException("分组名称不能重复");
            }
        }

        Integer specialServiceId = groupListVO.getSpecialServiceId();

        for (int i = 0; i < groupList.size(); i++) {
            GroupListVO.GroupListElement groupListElement = groupList.get(i);
            String groupName = groupListElement.getGroupName();
            Integer groupSort = i + 1;
            List<CrossListElement> crossList = groupListElement.getCrossList();
            for (CrossListElement crossListElement : crossList) {
                String crossId = crossListElement.getCrossId();
                specialServiceCrossMapper.updateGroupName(crossId, groupName, specialServiceId, groupSort);
            }
        }
    }

    @Override
    @Transactional
    public void deleteGroup(GroupNameBO groupNameBO) {
        Integer specialServiceId = groupNameBO.getSpecialServiceId();
        String groupName = groupNameBO.getGroupName();
        // 如果删除分组之后还有未分组路口，则更改分组序号
        List<SpecialServiceCrossPO> specialServiceCrossPOList =
                specialServiceCrossMapper.selectNoGroup(specialServiceId, "未分组路口");
        if (CollectionUtil.isNotEmpty(specialServiceCrossPOList)) {
            Integer sort = specialServiceCrossPOList.get(0).getGroupSort();
            specialServiceCrossMapper.deleteGroupChangeSort(specialServiceId, groupName, sort);
        } else {
            specialServiceCrossMapper.deleteGroup(specialServiceId, groupName);
        }
    }

    @Override
    @Transactional
    public void autoUnlock(GroupListVO groupListVO) {
        Integer specialServiceId = groupListVO.getSpecialServiceId();
        SpecialServicePO specialServicePO = specialServiceMapper.selectById(specialServiceId);
        String plateNum = specialServicePO.getPlateNum();
        ConsumerHandler.updateCarplate(plateNum);

        List<GroupListVO.GroupListElement> groupList = groupListVO.getGroupList();
        for (GroupListVO.GroupListElement groupListElement : groupList) {
            List<CrossListElement> crossList = groupListElement.getCrossList();
            for (CrossListElement crossListElement : crossList) {
                String crossId = crossListElement.getCrossId();
                Integer autoUnlock = crossListElement.getAutoUnlock();
                Integer status = 1; // 将自动解锁路口的状态设为锁定
                specialServiceCrossMapper.updateAutoUnlock(crossId, autoUnlock, specialServiceId, status);
            }
        }

        List<BaseCrossInfoPO> crossList = new ArrayList<>();
        List<SpecialServiceCrossPO> unLockCrosses = specialServiceCrossMapper.selectByAutoUnlock();
        for (SpecialServiceCrossPO unLockCross : unLockCrosses) {
            String crossId = unLockCross.getCrossId();
            BaseCrossInfoPO baseCrossInfoPO = baseCrossInfoMapper.selectById(crossId);
            crossList.add(baseCrossInfoPO);
        }
        ConsumerHandler.updateCrossList(crossList);
    }

    private List<GroupListVO.GroupListElement> buildGroupList(SpecialServicePO specialServicePO) {
        List<GroupListVO.GroupListElement> groupList = new ArrayList<>();

        Integer specialServiceId = specialServicePO.getId();
        List<SpecialServiceCrossPO> specialServiceCrossList =
                specialServiceCrossMapper.selectBySpecialServiceId(specialServiceId);

        LinkedHashMap<Integer, List<SpecialServiceCrossPO>> map = specialServiceCrossList.stream()
                .collect(Collectors.groupingBy(
                        SpecialServiceCrossPO::getGroupSort, LinkedHashMap::new, Collectors.toList()));

        for (Map.Entry<Integer, List<SpecialServiceCrossPO>> entry : map.entrySet()) {
            SpecialServiceCrossPO specialServiceCrossPO = entry.getValue().get(0);
            String groupName = specialServiceCrossPO.getGroupName();
            Integer groupSort = specialServiceCrossPO.getGroupSort();
            GroupListVO.GroupListElement groupListElement = new GroupListVO.GroupListElement();
            groupListElement.setGroupName(groupName);

            List<SpecialServiceCrossPO> groupCrossList = map.get(groupSort);
            Collections.sort(groupCrossList, Comparator.comparing(SpecialServiceCrossPO::getSort));
            StringBuilder wktBuilder = new StringBuilder();
            List<SpecialServiceDetailVO.CrossListElement> crossList = new ArrayList<>();
            buildWktAndCrosslist(wktBuilder, crossList, groupCrossList, groupName, specialServiceCrossList);

            groupListElement.setWkt(wktBuilder.toString());
            groupListElement.setCrossList(crossList);
            groupList.add(groupListElement);
        }

        return groupList;
    }

    private void buildWktAndCrosslist(StringBuilder wktBuilder, List<CrossListElement> crossList,
                                      List<SpecialServiceCrossPO> groupCrossList, String groupName,
                                      List<SpecialServiceCrossPO> specialServiceCrossList) {

        int crossNums = specialServiceCrossList.size();
        for (int i = 0; i < groupCrossList.size(); i++) {
            SpecialServiceCrossPO currentCrossElement = groupCrossList.get(i);
            // 构造wkt
            if (!"未分组路口".equals(groupName) && i != groupCrossList.size() - 1) { // 有分组且不是组内最后一个路口
                SpecialServiceCrossPO nextCrossElement = groupCrossList.get(i + 1);
                String nextCrossId = nextCrossElement.getCrossId();
                String currentCrossId = currentCrossElement.getCrossId();
                RidInfoEntity ridInfoEntity = ridInfoMapper.selectByStartEnd(currentCrossId, nextCrossId);
                String wktElement = ridInfoEntity.getWkt();
                wktBuilder.append(wktElement).append(";");
            }
            // 构造crossList
            String currentCrossId = currentCrossElement.getCrossId();
            BaseCrossInfoPO currentCross = baseCrossInfoMapper.selectById(currentCrossId);
            CrossListElement crossListElement = new CrossListElement();
            crossListElement.setCrossName(currentCross.getName());
            crossListElement.setCrossId(currentCrossId);

            Integer sort = currentCrossElement.getSort();
            if (sort == 1) {
                crossListElement.setRouteStart("起点");
                SpecialServiceCrossPO nextCross = specialServiceCrossList.get(sort);
                String nextCrossId = nextCross.getCrossId();
                // 构建路名+方向
                RidInfoEntity ridInfoEntity = ridInfoMapper.selectByStartEnd(currentCrossId, nextCrossId);
                String name = ridInfoEntity.getName(); // 西湖路:湘江中路@书院路路段
                String[] split = name.split(":");
                String streetName = split[0];
                Integer outDir = ridInfoEntity.getOutDir();
                String ourDirMsg = CrossDirEnum.getMsgByCode(outDir);
                crossListElement.setRouteEnd(streetName + ourDirMsg);
            } else if (sort == crossNums) { // 结束路口
                SpecialServiceCrossPO preCross = specialServiceCrossList.get(sort - 2);
                String preCrossId = preCross.getCrossId();
                // 构建路名+方向
                RidInfoEntity ridInfoEntity = ridInfoMapper.selectByStartEnd(preCrossId, currentCrossId);
                String name = ridInfoEntity.getName(); // 西湖路:湘江中路@书院路路段
                String[] split = name.split(":");
                String streetName = split[0];
                Integer inDir = ridInfoEntity.getInDir();
                String inDirMsg = CrossDirEnum.getMsgByCode(inDir);
                crossListElement.setRouteStart(streetName + inDirMsg);
                crossListElement.setRouteEnd("终点");
            } else { // 中间路口
                SpecialServiceCrossPO preCross = specialServiceCrossList.get(sort - 2);
                SpecialServiceCrossPO nextCross = specialServiceCrossList.get(sort);
                String preCrossId = preCross.getCrossId();
                String nextCrossId = nextCross.getCrossId();
                // 路线起点
                RidInfoEntity ridInfoEntity = ridInfoMapper.selectByStartEnd(preCrossId, currentCrossId);
                String name = ridInfoEntity.getName(); // 西湖路:湘江中路@书院路路段
                String[] split = name.split(":");
                String streetName = split[0];
                Integer inDir = ridInfoEntity.getInDir();
                String inDirMsg = CrossDirEnum.getMsgByCode(inDir);
                crossListElement.setRouteStart(streetName + inDirMsg);
                // 路线终点
                RidInfoEntity ridInfoEntity2 = ridInfoMapper.selectByStartEnd(currentCrossId, nextCrossId);
                String name2 = ridInfoEntity2.getName(); // 西湖路:湘江中路@书院路路段
                String[] split2 = name2.split(":");
                String streetName2 = split2[0];
                Integer outDir = ridInfoEntity2.getOutDir();
                String ourDirMsg = CrossDirEnum.getMsgByCode(outDir);
                crossListElement.setRouteEnd(streetName2 + ourDirMsg);
            }
            crossListElement.setStatus(currentCrossElement.getStatus());
            crossListElement.setAutoUnlock(currentCrossElement.getAutoUnlock());
            crossListElement.setSort(currentCrossElement.getSort());
            crossList.add(crossListElement);
        }
        // 去wkt掉最后一个分号
        if (wktBuilder.length() > 0) {
            wktBuilder.setLength(wktBuilder.length() - 1);
        }
    }

    private List<SpecialServiceDetailVO.CrossListElement> buildCrossList(Integer specialServiceId) {
        List<SpecialServiceDetailVO.CrossListElement> crossList = new ArrayList<>();
        List<SpecialServiceCrossPO> specialServiceCrossList =
                specialServiceCrossMapper.selectBySpecialServiceId(specialServiceId);

        for (int i = 0; i < specialServiceCrossList.size(); i++) {
            SpecialServiceCrossPO currentCross = specialServiceCrossList.get(i);
            String currentCrossId = currentCross.getCrossId();
            SpecialServiceDetailVO.CrossListElement crossListElement = new SpecialServiceDetailVO.CrossListElement();

            String crossId = currentCross.getCrossId();
            BaseCrossInfoPO baseCrossInfoPO = baseCrossInfoMapper.selectById(crossId);
            crossListElement.setCrossName(baseCrossInfoPO.getName());
            crossListElement.setCrossId(crossId);

            setElementStartEnd(specialServiceCrossList, i, currentCrossId, crossListElement);

            crossListElement.setStatus(currentCross.getStatus());
            crossListElement.setAutoUnlock(currentCross.getAutoUnlock());
            crossListElement.setSort(currentCross.getSort());

            String location = baseCrossInfoPO.getLocation();
            double[] lonLat = CrossUtil.getLonLat(location);
            crossListElement.setLng(lonLat[0]);
            crossListElement.setLat(lonLat[1]);

            crossList.add(crossListElement);
        }

        return crossList;
    }

    private void setElementStartEnd(List<SpecialServiceCrossPO> specialServiceCrossList, int i, String currentCrossId, CrossListElement crossListElement) {
        if (i == 0) { // 开始路口
            crossListElement.setRouteStart("起点");
            SpecialServiceCrossPO nextCross = specialServiceCrossList.get(i + 1);
            String nextCrossId = nextCross.getCrossId();
            // 构建路名+方向
            RidInfoEntity ridInfoEntity = ridInfoMapper.selectByStartEnd(currentCrossId, nextCrossId);
            String name = ridInfoEntity.getName(); // 西湖路:湘江中路@书院路路段
            String[] split = name.split(":");
            String streetName = split[0];
            Integer outDir = ridInfoEntity.getOutDir();
            String ourDirMsg = CrossDirEnum.getMsgByCode(outDir);
            crossListElement.setRouteEnd(streetName + ourDirMsg);
        } else if (i == specialServiceCrossList.size() - 1) { // 结束路口
            crossListElement.setRouteEnd("终点");
            SpecialServiceCrossPO preCross = specialServiceCrossList.get(i - 1);
            String preCrossId = preCross.getCrossId();
            // 构建路名+方向
            RidInfoEntity ridInfoEntity = ridInfoMapper.selectByStartEnd(preCrossId, currentCrossId);
            String name = ridInfoEntity.getName(); // 西湖路:湘江中路@书院路路段
            String[] split = name.split(":");
            String streetName = split[0];
            Integer inDir = ridInfoEntity.getInDir();
            String inDirMsg = CrossDirEnum.getMsgByCode(inDir);
            crossListElement.setRouteStart(streetName + inDirMsg);
        } else { // 中间路口
            SpecialServiceCrossPO preCross = specialServiceCrossList.get(i - 1);
            SpecialServiceCrossPO nextCross = specialServiceCrossList.get(i + 1);
            String preCrossId = preCross.getCrossId();
            String nextCrossId = nextCross.getCrossId();
            // 路线起点
            RidInfoEntity ridInfoEntity = ridInfoMapper.selectByStartEnd(preCrossId, currentCrossId);
            String name = ridInfoEntity.getName(); // 西湖路:湘江中路@书院路路段
            String[] split = name.split(":");
            String streetName = split[0];
            Integer inDir = ridInfoEntity.getInDir();
            String inDirMsg = CrossDirEnum.getMsgByCode(inDir);
            crossListElement.setRouteStart(streetName + inDirMsg);
            // 路线终点
            RidInfoEntity ridInfoEntity2 = ridInfoMapper.selectByStartEnd(currentCrossId, nextCrossId);
            String name2 = ridInfoEntity2.getName(); // 西湖路:湘江中路@书院路路段
            String[] split2 = name2.split(":");
            String streetName2 = split2[0];
            Integer outDir = ridInfoEntity2.getOutDir();
            String ourDirMsg = CrossDirEnum.getMsgByCode(outDir);
            crossListElement.setRouteEnd(streetName2 + ourDirMsg);
        }
    }

    @NotNull
    private List<BaseCrossInfoPO> getRouteList(BaseCrossInfoPO startCross, BaseCrossInfoPO endCross) {
        String endCrossLonLatStr = getCrossLonLatStr(endCross);
        // 获取上游路口到终点距离最近的
        BaseCrossInfoPO nextStartCross = startCross;
        List<BaseCrossInfoPO> routeList = new ArrayList<>();
        while (ObjectUtil.notEqual(endCross.getId(), nextStartCross.getId())) {
            String nextCrossId = nextStartCross.getId();
            double minDistance = Double.MAX_VALUE;
            List<RidInfoEntity> ridUpCrossList = ridInfoMapper.selectUpCross(nextStartCross.getId());
            for (RidInfoEntity ridUpCross : ridUpCrossList) {
                String upCrossId = ridUpCross.getStartCrossId();
                BaseCrossInfoPO upCross = baseCrossInfoMapper.selectById(upCrossId);
                String upCrossLonLatStr = getCrossLonLatStr(upCross);
                double distance = gtsService.distance(endCrossLonLatStr, upCrossLonLatStr);
                minDistance = Math.min(minDistance, distance);
                if (minDistance == distance && !hasCross(routeList, upCross)) { // 已选过的路口不可再选
                    nextStartCross = upCross;
                }
            }
            if (ObjectUtil.equal(nextStartCross.getId(), nextCrossId)) {
                throw new RuntimeException("路网数据不连续");
            }
            routeList.add(nextStartCross);
        }
        routeList.add(0, startCross);
        return routeList;
    }

    private List<RouteElementVO> buildRoute(List<BaseCrossInfoPO> routeList) {
        List<RouteElementVO> route = new ArrayList<>();
        // 遍历到倒数第二个路口
        for (int i = 0; i < routeList.size() - 1; i++) {
            RouteElementVO routeElementVO = new RouteElementVO();
            BaseCrossInfoPO firstCross = routeList.get(i);
            BaseCrossInfoPO secondCross = routeList.get(i + 1);
            String firstLonLatStr = getCrossLonLatStr(firstCross);
            String secondLonLatStr = getCrossLonLatStr(secondCross);

            routeElementVO.setStartLonLat(firstLonLatStr);
            routeElementVO.setStartId(firstCross.getId());
            routeElementVO.setStartName(firstCross.getName());
            routeElementVO.setEndLonLat(secondLonLatStr);
            routeElementVO.setEndId(secondCross.getId());
            routeElementVO.setEndName(secondCross.getName());

            // 获取路况，1为进口
            Integer status = getRidStatus(firstCross, secondCross, 1);
            routeElementVO.setStatus(status);
            // 获取wkt
            String wkt = getWkt(firstCross, secondCross);
            routeElementVO.setWkt(wkt);

            route.add(routeElementVO);
        }
        return route;
    }

    private String getWkt(BaseCrossInfoPO firstCross, BaseCrossInfoPO secondCross) {
        String firstId = firstCross.getId();
        String secondId = secondCross.getId();
        RidInfoEntity ridInfoEntity = ridInfoMapper.selectByStartEnd(firstId, secondId);
        return ridInfoEntity.getWkt();
    }

    private Integer getRidStatus(BaseCrossInfoPO firstCross, BaseCrossInfoPO secondCross, int inOutType) {
        String firstId = firstCross.getId();
        String secondId = secondCross.getId();
        RidInfoEntity ridInfoEntity = ridInfoMapper.selectByStartEnd(firstId, secondId);
        Integer inDir = ridInfoEntity.getInDir();
        Integer status = crossDirDataRealtimeMapper.selectStatus(secondId, inDir, inOutType);
        return status;
    }

    private String getCrossLonLatStr(BaseCrossInfoPO firstCross) {
        String firstLocation = firstCross.getLocation();
        double[] firstLonLat = CrossUtil.getLonLat(firstLocation);
        return firstLonLat[0] + "," + firstLonLat[1];
    }
}